@@ -25,7 +25,7 @@ from ipywidgets import VBox, HBox, Button, Output, Text
 from .utils import (
     detect_robot_context, line_at_cursor, scored_results,
     complete_libraries, get_lunr_completions, remove_prefix,
-    display_log, process_screenshots
+    display_log, process_screenshots, lunr_query, get_keyword_doc
 )
 from .selectors import (
     BrokenOpenConnection, clear_selector_highlights, get_autoit_selector_completions, get_selector_completions,
@@ -35,6 +35,9 @@ from .selectors import (
 from .constants import VARIABLE_REGEXP, BUILTIN_VARIABLES
 from .listeners import RobotKeywordsIndexerListener
 
+from robot.running.model import UserKeyword
+UserKeyword.source = property(lambda self: self.actual_source)
+
 
 def normalize_argument(name):
     if "=" in name:
@@ -264,8 +267,16 @@ def _execute_impl(code: str, suite: TestSuite, defaults: TestDefaults = TestDefa
     strip_duplicate_items(suite.resource.variables)
     strip_duplicate_items(suite.resource.keywords)
 
+    new_imports = [item for item in get_items_copy(suite.resource.imports) if item not in imports]
+    for new_import in new_imports:
+        new_import.source = suite.source
+    new_variables = [item for item in get_items_copy(suite.resource.variables) if item not in variables]
+    for new_variable in new_variables:
+        new_variable.source = suite.source
     # If there is no test, allow the user to interact with defined keywords by providing widgets
     new_keywords = [item for item in get_items_copy(suite.resource.keywords) if item not in keywords]
+    for new_keyword in new_keywords:
+        new_keyword.actual_source = suite.source
     if not suite.tests and new_keywords and interactive_keywords:
         return None, [get_interactive_keyword(suite, keyword) for keyword in new_keywords]
 
@@ -398,6 +409,39 @@ def complete(code: str, cursor_pos: int, suite: TestSuite, keywords_listener: Ro
     }
 
 
+def inspect(code: str, cursor_pos: int, suite: TestSuite, keywords_listener: RobotKeywordsIndexerListener = None, detail_level=0):
+    cursor_pos = len(code) if cursor_pos is None else cursor_pos
+    line, offset = line_at_cursor(code, cursor_pos)
+    line_cursor = cursor_pos - offset
+    left_needle = re.split(r"\s{2,}|\t| \| ", line[:line_cursor])[-1]
+    right_needle = re.split(r"\s{2,}|\t| \| ", line[line_cursor:])[0]
+    needle = left_needle.lstrip().lower() + right_needle.rstrip().lower()
+
+    results = []
+    data = {}
+    found = False
+
+    if needle and lunr_query(needle):
+        query = lunr_query(needle)
+        results = keywords_listener.index.search(query)
+        results += keywords_listener.index.search(query.strip("*"))
+
+    for result in results:
+        keyword = keywords_listener.keywords[result["ref"]]
+
+        if needle not in [keyword.name.lower(), result["ref"].lower()]:
+            continue
+
+        data = get_keyword_doc(keyword)
+        found = True
+        break
+
+    return {
+        "data": data,
+        "found": found,
+    }
+
+
 def shutdown_drivers(drivers=[]):
     for driver in drivers:
         if hasattr(driver["instance"], "quit"):
